"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFileOutlineHighlights = exports.highlightsThatFit = exports.NumCharsSizer = void 0;
const lodash_1 = __importDefault(require("lodash"));
const ranker_1 = require("./ranker");
const highlighter_1 = require("./highlighter");
const outliner_1 = require("./outliner");
class NumCharsSizer {
    constructor(sizeInChars) {
        this.sizeInChars = sizeInChars;
    }
    fits(content) {
        return content.length <= this.sizeInChars;
    }
}
exports.NumCharsSizer = NumCharsSizer;
function highlightsThatFit(contextSizer, charSourceSet, otherSources, contentPath) {
    return __awaiter(this, void 0, void 0, function* () {
        let loPercentile = 0;
        let hiPercentile = 1;
        let percentile = 1;
        while (true) {
            const highlights = yield generateSourceSetHighlights(percentile, charSourceSet, otherSources, contentPath);
            if (!highlights) {
                loPercentile = percentile;
            }
            else {
                const fits = contextSizer.fits(highlights);
                loPercentile = fits ? percentile : loPercentile;
                hiPercentile = fits ? hiPercentile : percentile;
            }
            const nextPercentile = (loPercentile + hiPercentile) / 2;
            if (Math.abs(nextPercentile - percentile) < 0.001)
                return highlights;
            percentile = nextPercentile;
        }
    });
}
exports.highlightsThatFit = highlightsThatFit;
function generateSourceSetHighlights(topPercentile, chatSources, otherSources, contentPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (topPercentile <= 0 || 1 < topPercentile)
            throw new Error('topPercentile must be between 0 and 1');
        const allSources = [...chatSources, ...otherSources];
        const rankedTags = yield (0, ranker_1.createRankedTags)({ sources: allSources, contentPath: contentPath });
        if (!rankedTags)
            return;
        const chatRelPaths = chatSources.map(source => source.relPath);
        const topTags = rankedTags.without(chatRelPaths).toTags();
        const maxTags = lodash_1.default.round(topPercentile * topTags.length);
        const mapTags = topTags.slice(0, maxTags);
        const sortedTags = lodash_1.default.orderBy(mapTags, ['relPath', 'start.ln'], ['asc', 'asc']);
        const groupedTags = lodash_1.default.groupBy(sortedTags, tag => tag.relPath);
        const fileHighlights = yield Promise.all(lodash_1.default.values(groupedTags).map(tags => {
            const relPath = tags[0].relPath;
            const code = allSources.find(source => source.relPath === relPath).code;
            return (0, highlighter_1.generateFileHighlightsFromTags)(tags, code, contentPath);
        }));
        return lodash_1.default.join(fileHighlights, '');
    });
}
function generateFileOutlineHighlights(sourceSet) {
    return __awaiter(this, void 0, void 0, function* () {
        const outlines = yield (0, outliner_1.createOutlines)(sourceSet);
        if (!outlines)
            return;
        const fileOutlines = yield Promise.all(lodash_1.default.zip(outlines, sourceSet.sources)
            .filter(([tags, _sources]) => {
            return tags;
        })
            .map(([tags, source]) => {
            return (0, outliner_1.generateFileOutlineFromTags)(tags, source.code);
        }));
        return lodash_1.default.join(fileOutlines, '');
    });
}
exports.generateFileOutlineHighlights = generateFileOutlineHighlights;
//# sourceMappingURL=index.shared.js.map